
# q-expansion of newform 162.5.d.d, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 162
weight = 5
poly_data = [1, 0, 0, 0, -1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 0, 0, 1, 0, 0, 0], 1], [[0, 0, 3, 0, 0, 0, 3, 0], 1], [[0, 1, 0, 0, 0, 0, 0, -1], 1], [[0, 0, 6, 0, 0, 0, -3, 0], 1], [[0, 0, 0, 1, 0, 1, 0, -1], 1], [[0, 3, 0, 2, 0, -1, 0, 1], 1], [[0, 1, 0, -1, 0, 2, 0, 3], 1]]

hecke_ring_character_values = [[83, [1, -1, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, -2, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 22, -5, 0], [0, 13, -13, 0, 0, 0, 0, 0], [0, 0, 0, 59, 0, 0, -10, 10], [-143, 143, 14, 0, -14, 0, 0, 0], [0, 0, 0, 116, 0, -116, 0, 31], [-31, 0, 0, 0, -43, 0, 0, 0], [0, 0, 0, 0, 0, 35, 200, 0], [0, 0, 0, -37, 0, 0, -295, 295], [-896, 896, 54, 0, -54, 0, 0, 0], [-2350, 0, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 0, -203, -266, 0], [0, -755, -177, 0, 0, 0, 0, 0], [0, 0, 0, -510, 0, 0, -594, 594], [0, 0, 0, 421, 0, -421, 0, -262], [0, 0, 0, 0, 0, 530, -358, 0], [0, 1036, -351, 0, 0, 0, 0, 0], [3019, -3019, 1131, 0, -1131, 0, 0, 0], [0, 0, 0, -3757, 0, 3757, 0, 592], [-448, 0, 0, 0, 1455, 0, 0, 0], [0, 3751, -53, 0, 0, 0, 0, 0], [0, 0, 0, 450, 0, 0, -408, 408], [0, 0, 0, -1309, 0, 1309, 0, 2041], [0, -11576, 310, 0, 0, 0, 0, 0], [0, 0, 0, -1391, 0, 0, 2704, -2704], [8392, -8392, -984, 0, 984, 0, 0, 0], [0, 0, 0, 8824, 0, -8824, 0, 2846], [11135, 0, 0, 0, 1500, 0, 0, 0], [0, 0, 0, 0, 0, -5424, 405, 0], [-17863, 0, 0, 0, -453, 0, 0, 0], [0, 0, 0, 0, 0, 1717, -3764, 0], [0, 0, 0, -13345, 0, 0, 941, -941], [-650, 650, -2846, 0, 2846, 0, 0, 0], [0, 0, 0, 0, 0, -5515, 4271, 0], [0, 14608, -3348, 0, 0, 0, 0, 0], [13342, -13342, 2091, 0, -2091, 0, 0, 0], [-7228, 0, 0, 0, -5450, 0, 0, 0], [0, 0, 0, 0, 0, 9663, 8976, 0], [0, 0, 0, 32859, 0, 0, 1581, -1581], [0, 0, 0, -3662, 0, 3662, 0, 6224], [-39646, 0, 0, 0, -2066, 0, 0, 0], [0, 0, 0, 20587, 0, 0, -7160, 7160], [-10283, 10283, 2646, 0, -2646, 0, 0, 0], [0, 0, 0, 15785, 0, -15785, 0, -10157], [10364, 0, 0, 0, -3616, 0, 0, 0], [40429, -40429, 1295, 0, -1295, 0, 0, 0], [0, 5239, 2943, 0, 0, 0, 0, 0], [0, 0, 0, 29349, 0, 0, 3894, -3894], [13711, -13711, 1236, 0, -1236, 0, 0, 0], [0, 0, 0, 4165, 0, -4165, 0, 4445], [0, 0, 0, 0, 0, -45444, 8760, 0], [0, -10400, 3803, 0, 0, 0, 0, 0], [0, 0, 0, -24347, 0, 24347, 0, 17144], [0, 0, 0, 0, 0, -37251, -9921, 0], [0, 0, 0, 9016, 0, 0, -6050, 6050], [0, 0, 0, 60034, 0, -60034, 0, 4511], [60653, 0, 0, 0, -9831, 0, 0, 0], [0, 10036, -10464, 0, 0, 0, 0, 0], [0, 0, 0, -17962, 0, 0, 33521, -33521], [121090, -121090, 906, 0, -906, 0, 0, 0], [0, 0, 0, 0, 0, -15676, -31189, 0], [36686, 0, 0, 0, 12216, 0, 0, 0], [0, 0, 0, 0, 0, 69322, -10058, 0], [0, -42539, -6346, 0, 0, 0, 0, 0], [0, 0, 0, 2566, 0, 0, 14365, -14365], [0, 9493, 11533, 0, 0, 0, 0, 0], [19294, -19294, 20800, 0, -20800, 0, 0, 0], [0, 0, 0, 0, 0, -113043, 11442, 0], [0, -121424, -3340, 0, 0, 0, 0, 0], [0, 0, 0, -90001, 0, 0, 11288, -11288], [0, 0, 0, -46575, 0, 46575, 0, 29538], [0, 83080, -32502, 0, 0, 0, 0, 0], [-70172, 70172, -11134, 0, 11134, 0, 0, 0], [-61588, 0, 0, 0, -29750, 0, 0, 0], [0, 0, 0, 0, 0, 185197, -5522, 0], [0, 0, 0, 5377, 0, 0, -44258, 44258], [1622, 0, 0, 0, 651, 0, 0, 0], [0, 0, 0, 0, 0, -33077, -29603, 0], [-11000, 11000, 34201, 0, -34201, 0, 0, 0], [0, 0, 0, 0, 0, 143274, -43506, 0], [0, 121711, -12724, 0, 0, 0, 0, 0], [0, 0, 0, 17798, 0, -17798, 0, -6140], [-121651, 0, 0, 0, -16110, 0, 0, 0], [0, 38080, 55134, 0, 0, 0, 0, 0], [0, 0, 0, -19542, 0, 0, 45570, -45570], [0, 0, 0, -23979, 0, 23979, 0, -35502], [0, -148544, 19629, 0, 0, 0, 0, 0], [0, 0, 0, -91285, 0, 0, -17158, 17158], [-167660, 167660, -35590, 0, 35590, 0, 0, 0], [0, 0, 0, 46534, 0, -46534, 0, -35116], [0, 0, 0, 4065, 0, 0, 19470, -19470], [-53410, 0, 0, 0, 38492, 0, 0, 0], [0, 0, 0, 0, 0, -90799, 18248, 0], [106735, -106735, -27189, 0, 27189, 0, 0, 0], [0, 0, 0, 57648, 0, -57648, 0, 129606], [0, 0, 0, 0, 0, 14623, 60190, 0], [0, 0, 0, -87721, 0, 87721, 0, 28378], [-68461, 0, 0, 0, 35031, 0, 0, 0], [-127819, 0, 0, 0, 37840, 0, 0, 0], [0, 63712, 14970, 0, 0, 0, 0, 0], [0, 0, 0, 22949, 0, -22949, 0, 20911], [0, 0, 0, 0, 0, -163620, -41898, 0], [0, 0, 0, 175978, 0, 0, 3709, -3709], [-105914, 105914, -26684, 0, 26684, 0, 0, 0], [296327, 0, 0, 0, -30762, 0, 0, 0], [0, 0, 0, -333947, 0, 0, 42046, -42046], [0, 0, 0, 174149, 0, -174149, 0, 118471], [0, 0, 0, 0, 0, -146266, 65390, 0], [0, -98495, -4092, 0, 0, 0, 0, 0], [243133, -243133, 68815, 0, -68815, 0, 0, 0], [-193768, 0, 0, 0, -77970, 0, 0, 0], [0, 0, 0, 0, 0, 209185, -27803, 0], [0, 614386, 23568, 0, 0, 0, 0, 0], [-45352, 0, 0, 0, 3688, 0, 0, 0], [0, 0, 0, -294427, 0, 0, 4457, -4457], [-218300, 218300, -54396, 0, 54396, 0, 0, 0], [0, 0, 0, -78789, 0, 78789, 0, -21732], [0, 0, 0, 0, 0, -9535, -22258, 0], [0, 0, 0, -45275, 0, 0, -150296, 150296], [-357242, 357242, -68127, 0, 68127, 0, 0, 0], [0, -254801, -81902, 0, 0, 0, 0, 0], [0, 0, 0, 85279, 0, 0, -46490, 46490], [0, 0, 0, -167892, 0, 167892, 0, -55650], [0, -264293, -35559, 0, 0, 0, 0, 0], [0, 0, 0, -273272, 0, 273272, 0, 4943], [0, 892447, -7844, 0, 0, 0, 0, 0], [0, 0, 0, 195579, 0, -195579, 0, -154026], [0, -183239, -122877, 0, 0, 0, 0, 0], [-50054, 50054, -96222, 0, 96222, 0, 0, 0], [598340, 0, 0, 0, -12504, 0, 0, 0], [0, 0, 0, 0, 0, 30816, 123324, 0], [-460649, 460649, 25399, 0, -25399, 0, 0, 0], [250220, 0, 0, 0, -53964, 0, 0, 0], [0, 0, 0, 0, 0, -545403, 106149, 0], [-124145, 124145, 147074, 0, -147074, 0, 0, 0], [0, 0, 0, 259497, 0, -259497, 0, 187947], [-496463, 496463, 8277, 0, -8277, 0, 0, 0], [0, 0, 0, 0, 0, 259995, -198675, 0], [0, 0, 0, -395299, 0, 395299, 0, 32017], [4520, 0, 0, 0, 141768, 0, 0, 0], [0, 0, 0, -553322, 0, 0, 163069, -163069], [-239786, 239786, 79298, 0, -79298, 0, 0, 0], [0, 0, 0, -46669, 0, 46669, 0, -23384], [184316, 0, 0, 0, -20368, 0, 0, 0], [0, 0, 0, -245115, 0, 0, 63402, -63402], [0, 57286, -169752, 0, 0, 0, 0, 0], [0, 0, 0, 98133, 0, 0, -179547, 179547], [1228948, -1228948, -16586, 0, 16586, 0, 0, 0], [0, 0, 0, 100085, 0, -100085, 0, 322900], [-37352, 37352, -142871, 0, 142871, 0, 0, 0], [0, 0, 0, 47989, 0, -47989, 0, 186818], [-1014502, 0, 0, 0, 72410, 0, 0, 0], [0, 0, 0, 0, 0, -78905, 285100, 0], [0, 542320, 146626, 0, 0, 0, 0, 0], [0, 0, 0, 950457, 0, 0, -60066, 60066], [159743, 0, 0, 0, 76755, 0, 0, 0], [0, 0, 0, -236538, 0, 0, -125121, 125121], [22049, 0, 0, 0, 167746, 0, 0, 0], [0, 0, 0, 0, 0, 86147, 79913, 0], [0, 0, 0, 185835, 0, 0, 105804, -105804], [0, 0, 0, 160065, 0, -160065, 0, 61155], [-39263, 39263, -238605, 0, 238605, 0, 0, 0], [0, 0, 0, 54879, 0, -54879, 0, -297816], [0, 0, 0, 0, 0, 617399, -166498, 0], [0, 0, 0, 770932, 0, 0, -119150, 119150], [-698785, 0, 0, 0, -42717, 0, 0, 0], [0, 12034, -195039, 0, 0, 0, 0, 0]]
